"""
Created on 26 mars 2018

@author: Tonneau Stephane
"""
import os
import psutil
from pidclient.base_utils import first_not_null
from pidclient.base_utils import Configuration
from pidclient.base_utils import SystemEnvironment

DEFAULTS = {"driver_memory": None, "executor_memory": None,
            "min_executors": None, "max_executors": None,
            "num_executors": None, "init_executors": None,
            "queue" : None, "executor_environment" : None}

SPARK_CDRIVER_PTYPE = "spark_cdriver"
SPARK_DRIVER_PTYPE = "spark_driver"
LOCAL_EXECUTOR_PTYPE = "local_executor"

def get_conf_value(conf, key):
    """
    Get a specific SparkConf value.  This function return None when
    conf object is None.

    :param conf: configuration of the spark environment
    :type conf: SparkConf
    :param key: the whished key value
    :type key: string
    """
    if conf is None:
        return None
    return conf.get(key)

class YarnClientConfiguration(Configuration):
    """
    Class that encapsulates a Spark Configuration in client mode.
    """

    def complete(self, conf):
        """
        Complete the configuration information with
        the environments definitions (or default when not available)
        for spark configuration running in client mode.

        :param conf: spark configuration within the client spark environment
        :type conf: pyspark.SparkConf
        """

        self.driver_memory = first_not_null(self.driver_memory,
                                            get_conf_value(conf, "spark.yarn.am.memory"),
                                            DEFAULTS["driver_memory"])
        self.executor_memory = first_not_null(self.executor_memory, DEFAULTS["executor_memory"])
        self.min_executors = first_not_null(self.min_executors, DEFAULTS["min_executors"])
        self.max_executors = first_not_null(self.max_executors,
                                            get_conf_value(conf,
                                                           "spark.yarn.containerLauncherMaxThreads"),
                                            DEFAULTS["max_executors"])
        self.num_executors = first_not_null(self.num_executors, DEFAULTS["num_executors"])
        self.init_executors = first_not_null(self.init_executors, DEFAULTS["init_executors"])
        self.queue = first_not_null(self.queue, DEFAULTS["queue"])
        self.executor_environment = first_not_null(
            self.executor_environment, DEFAULTS["executor_environment"])
        return self


class YarnEnvironment(SystemEnvironment):
    """
    Class that captures Spark environment particularities (for client mode)
    """

    def __init__(self, **configuration):
        """Initialisator of the class

        :param configuration: specific parameters of the local Spark environment
        :type configuration: a dictionary
        """

        super(YarnEnvironment, self).__init__(**configuration)
        if "SPARK_LOCAL_DIRS" in os.environ:
            # This is a slave ... but we shouldn't be there
            process = psutil.Process().parent().parent().parent().parent()
            self.job_type = LOCAL_EXECUTOR_PTYPE
        else:
            # This is the master
            self.configuration = YarnClientConfiguration()
            conf = None
            self.job_type = SPARK_CDRIVER_PTYPE
            try:
                from pyspark import SparkConf
                conf = SparkConf(True)
            except Exception:
                # We can not get the required configuration
                # We will ignore this.
                pass
            self.configuration.complete(conf)
            process = psutil.Process().parent().parent().parent()
        if (configuration is not None) and ('application_id' in configuration) and (self.job_type == SPARK_CDRIVER_PTYPE):
            self.job_id = configuration['application_id']
            self.main_id = self.job_id[12:]
            if self.job_type == SPARK_CDRIVER_PTYPE:
                self.job_type = SPARK_DRIVER_PTYPE
        else:
            node_str = self.get_node()
            self.job_id = node_str+"."+str(process.pid) + "." + str(process.create_time())
            self.main_id = node_str+"_"+ str(os.getpid()) + "_" + \
                str(process.create_time()).replace(".", "_")
        return
