"""
Created on 26 mars 2018

@author: Stephane Tonneau

This module requires the installation of the python kafka library with
    pip install kafka-python

"""
import sys
from pidclient.base_utils import LoggingMethod, JsonBased
try:
    import gssapi
except ImportError:
    pass
# The next line can be required to avoid the import of snappy as VITO
# has also a snappy package from ESA (with different functionalities).
# The side effect of the ESA package is that it induces a 'too many file open'
# issue for kafka producer.
# sys.modules['snappy'] = None
try:
    from kafka import KafkaProducer
except ImportError:
    # the unavailibility of the package shall be treated later
    # when trying to use a producer
    pass

# Topic writing in kafka need a list of bytes
# in python 3.x, the function is bytes(source,'utf-8')
# while bytes(source) is required in python 2.x
#
try:
    THE_BYTES = bytes('xxx')
except:

    def to_topic(source):
        """ return the string as a byte encoded list of characters for python 3.x"""
        return bytes(source, 'utf-8')

else:

    def to_topic(source):
        """ return the string as a byte encoded list of characters for python 2.x"""
        return bytes(source)

NODENAME = "kafka"

class KafkaMessage(JsonBased):
    """Class that encapsulate some metainformation of pid entries"""

    def __init__(self, es_id, content):
        """
        Initialisator of the class

        :param content: The content of the message
        :type content: JsonBased
        """

        self.content = content
        "The content of the message"
        self.id = es_id
        "The id of the message"
        self.es_index = None
        "An optional Elastic Search index info"
        self.es_type = None
        "An optional Elastic Search type info"


class KafkaLogging(LoggingMethod):
    """Class that encapsulates the storing of objects in a kafka topic
    for logging purposes.
    """

    def __init__(self, **parameters):
        """Initialisator of the class

        :param configuration: the general logging configuration
        :type configuration: a python configuration object
        """
        super(KafkaLogging, self).__init__(**parameters)
        self.es_index = self.get_config(NODENAME, "es_index")
        self.es_type = self.get_config(NODENAME, "es_type")

        kafka_brokers = self.get_config(NODENAME, "brokers")
        if kafka_brokers is None:
            kafka_brokers = "epod-master1.vgt.vito.be:6668,epod-master2.vgt.vito.be:6668,epod-master3.vgt.vito.be:6668"

        self.kafka_brokers = kafka_brokers.replace(" ", "").split(",")
        self.kafka_topic = self.get_config(NODENAME, "topic")
        if self.kafka_topic is None:
            self.kafka_topic = 'pid-2-es'

        self.secure = self.get_config(NODENAME, "secure")
        try:
            if self.secure is None:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.kafka_brokers, acks='all')
            else:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.kafka_brokers, security_protocol='SASL_PLAINTEXT',
                    sasl_mechanism='GSSAPI', acks='all')
            if self.producer is None:
                self.py_log.error("Kafka producer NOT created!")
        except (Exception) as error:
            self.py_log.error("Unable to create a Kafka producer for brokers {}, topic {}, mode {}: {}"
                              .format(str(self.kafka_brokers), str(self.kafka_topic), str(self.secure), str(error)))
            self.producer = None

    def close(self):
        """Close the logging instance"""
        if self.producer is not None:
            try:
                self.producer.flush()
                self.producer.close()
            except:
                self.py_log.error("Unable to close Kafka producer for brokers {}, topic {}, mode {}".format(
                                  str(self.kafka_brokers), str(self.kafka_topic), self.secure))
            self.producer = None

    def process_log(self, entry, meta_information):
        """Method to insert a json representation of both an object
        and its metadata in a kafka topic

        :param entry: the standard information to log
        :type entry: PidEntry
        :param meta_information: meta information about the information to log
        :type meta_information: Metainformation
        """
        res = None
        if self.producer is not None:
            try:
                message = KafkaMessage(meta_information.uid, entry)
                if self.es_index is not None:
                    message.es_index = self.es_index
                if self.es_type is not None:
                    message.es_type = self.es_type
                future = self.producer.send(
                    self.kafka_topic, to_topic(str(message.to_json())))
                res = future.get(timeout=60)
            except (Exception) as error:
                self.py_log.error("Unable to publish Kafka message for brokers {}, topic {}, mode {}: {}".format(
                                  str(self.kafka_brokers), str(self.kafka_topic), str(self.secure), str(error)))
        return res
